@@ -20,5 +20,5 @@ To search for the existence of files, use the `find_file()` function.
 - since 4.8.0
 DOC
 ) do |args|
-  Error.is4x('binary_file')
+  Puppet::Parser::Functions::Error.is4x('binary_file')
 end
