@@ -19,7 +19,7 @@ class Puppet::Provider::Command
   # @param args [Array<String>] Any command line arguments to pass to the executable
   # @return The output from the command
   def execute(*args)
-    resolved_executable = @resolver.which(@executable) or raise Puppet::MissingCommand, "Command #{@name} is missing"
+    resolved_executable = @resolver.which(@executable) or raise Puppet::MissingCommand, _("Command %{name} is missing") % { name: @name }
     @executor.execute([resolved_executable] + args, @options)
   end
 end
