@@ -1,4 +1,4 @@
-require 'puppet/util/platform'
+require_relative '../../puppet/util/platform'
 
 # This file should be required instead of writing `require 'openssl'`
 # or any library that loads openssl like `net/https`. This allows the
@@ -21,8 +21,4 @@ else
       class Certificate; end
     end
   end
-
-  Puppet::SSL::Certificate.indirection.terminus = :none
-  Puppet::SSL::CertificateRequest.indirection.terminus = :none
-  Puppet::SSL::Key.indirection.terminus = :none
 end
