@@ -1,5 +1,5 @@
-require 'puppet/util/checksums'
-require 'puppet/util/diff'
+require_relative '../../../puppet/util/checksums'
+require_relative '../../../puppet/util/diff'
 require 'date'
 require 'tempfile'
 
@@ -42,7 +42,11 @@ module Puppet
           send resource[:loglevel], "[diff redacted]"
         else
           write_temporarily(param) do |path|
-            send resource[:loglevel], "\n" + diff(resource[:path], path)
+            diff_output = diff(resource[:path], path)
+            if diff_output.encoding == Encoding::BINARY || !diff_output.valid_encoding?
+              diff_output = "Binary files #{resource[:path]} and #{path} differ"
+            end
+            send resource[:loglevel], "\n" + diff_output
           end
         end
       end
@@ -75,7 +79,7 @@ module Puppet
       return :absent unless stat
       ftype = stat.ftype
       # Don't even try to manage the content on directories or links
-      return nil if ["directory","link"].include?(ftype)
+      return nil if ['directory', 'link', 'fifo', 'socket'].include?(ftype)
 
       begin
         resource.parameter(:checksum).sum_file(resource[:path])
