@@ -24,7 +24,11 @@ module Puppet::Util::Yaml
   # @raise [YamlLoadException] If deserialization fails.
   # @return The parsed YAML, which can be Hash, Array or scalar types.
   def self.safe_load(yaml, allowed_classes = [], filename = nil)
-    data = YAML.safe_load(yaml, allowed_classes, [], true, filename)
+    if Gem::Version.new(Psych::VERSION) >= Gem::Version.new('3.1.0')
+      data = YAML.safe_load(yaml, permitted_classes: allowed_classes, aliases: true, filename: filename)
+    else
+      data = YAML.safe_load(yaml, allowed_classes, [], true, filename)
+    end
     data = false if data.nil?
     data
   rescue ::Psych::DisallowedClass => detail
@@ -42,26 +46,15 @@ module Puppet::Util::Yaml
     safe_load(yaml, allowed_classes, filename)
   end
 
-  # @deprecated Use {#safe_load_file} instead.
-  def self.load_file(filename, default_value = false, strip_classes = false)
-    Puppet.deprecation_warning(_("Puppet::Util::Yaml.load_file is deprecated. Use safe_load_file instead."))
-
-    if(strip_classes) then
-      data = YAML::parse_file(filename)
-      data.root.each do |o|
-        if o.respond_to?(:tag=) and
-           o.tag != nil and
-           o.tag.start_with?("!ruby")
-          o.tag = nil
-        end
-      end
-      data.to_ruby || default_value
-    else
-      yaml = YAML.load_file(filename)
-      yaml || default_value
-    end
-  rescue *YamlLoadExceptions => detail
-    raise YamlLoadError.new(detail.message, detail)
+  # Safely load the content from a file as YAML if
+  # contents are in valid format. This method does not
+  # raise error but returns `nil` when invalid file is
+  # given.
+  def self.safe_load_file_if_valid(filename, allowed_classes = [])
+    safe_load_file(filename, allowed_classes)
+  rescue YamlLoadError, ArgumentError, Errno::ENOENT => detail
+    Puppet.debug("Could not retrieve YAML content from '#{filename}': #{detail.message}")
+    nil
   end
 
   def self.dump(structure, filename)
